require 'rhn/apigen'

include ApiGen

describe "Access Manager" do
  it "should register objects and provide access for them through a resolver" do

    # Given that I have a random python object declared somewhere in my
    # computer
    class RandomObject
      def get_stuff_1
        "blah"
      end

      def get_stuff_2
        "bleh"
      end
    end

    # When I allow the object and a method to be accessed by my crazy API
    AccessManager.register RandomObject, "get_stuff_1"

    # Then I see that now the registered method (get_stuff_1) is available but
    # the other one (get_stuff_2) remains protected
    Resolver.new("RandomObject.get_stuff_1").v.should be_eql("blah")
    expect { Resolver.new("RandomObject.get_stuff_2").v }.to raise_error AccessDenied, "You don't have access to this resource"
  end

  it "should return a smart message when the object does not exist" do
    expect {
      Resolver.new("NoSuchObject.no_such_method").v
    }.to raise_error NotFound, "There's no such object"
  end

  it "should provide access to attributes" do
    # Given that I have a random object with an attribute
    class RandomObject
      attr_accessor :attr

      def initialize
        @attr = 'random value'
      end
    end

    # When I register the attr through the AccessManager
    AccessManager.register RandomObject, 'attr'

    # Then I see that I can retrieve the right value
    Resolver.new('RandomObject.attr').v.should be_eql('random value')
  end

  it "should access method that takes parameters" do
    # Given that I have a class with a method that takes parameters
    class RandomObject
      def sum a, b
        a + b
      end
    end

    # When I register the attr in the AccessManager
    AccessManager.register RandomObject, 'sum'

    # Then I see that it's possible to pass arguments to my function
    Resolver.new('RandomObject.sum').v(2, 2).should be_eql(4)
  end

  it "should provide an easier way to resolve symbols" do
    # Given that I have a class with a method that takes parameters and
    # properly registered in the access manager
    class RandomObject
      def sum a, b
        a + b
      end
    end

    AccessManager.register RandomObject, 'sum'

    # When I query for the symbol using a dsl-ish description contained in a
    # dictionary, I see that I still get my results! :)
    ApiGen::from_dsl({:symbol => 'RandomObject.sum', :args => [2, 2]}).should be_eql(4)
  end
end
