require 'cgi'
require 'json'
require 'spec_helper'

require 'rhn/util'
require 'rhn/apigen'

describe "Web Access" do

  it "should be possible to encode requested symbols" do
    # Given that I have a request to the API described in the mini-dsl lang
    req = {"symbol" => 'MyObject', "args" => [1, 2]}

    # When I prepare the request
    prepared = encode_req(req)

    # Then I see that it was transformed into a json string encoded by the
    # ruby's `CGI::escape` function.
    JSON.parse(CGI::unescape(prepared)).should be_eql(req)
  end

  it "should be possible to decode requested symbols" do
    # Given that I have an encoded param
    encoded_req = \
        '%7B%22symbol%22%3A+%22MyObject%22%2C+%22args%22%3A+%5B1%2C+2%5D%7D'

    # When I decode the param above
    decoded = decode_req(encoded_req)

    # Then I see that it was transformed in a dict
    decoded.should be_eql({:symbol => 'MyObject', :args => [1, 2]})
  end

  it "should provide access for classes registered in the access manager" do

    # Given that I have an object registered in the AccessManager
    class Stuff
      def mul a, b
        a * b
      end
    end

    ApiGen::AccessManager.register Stuff, 'mul'

    # When I hit the API that resolves objects with the right parameters
    desc = encode_req({:symbol => 'Stuff.mul', :args => [2, 2]})
    get "/api/?r=#{desc}"

    # Then I see that the result was right
    last_response.status.should be_eql 200
    last_response.headers["content-type"].should be_eql "application/json;charset=utf-8"
    JSON.parse(last_response.body).should be_eql({'result' => 4})
  end

  it "should return an error code when the user tries to access denied resources" do
    # Given that I have an object registered in the AccessManager with
    # unregistered symbols
    class Calc
      def sum a, b
        a + b
      end
      def mul a, b
        a * b
      end
    end

    ApiGen::AccessManager.register Calc, 'mul'

    # When I try to access the unregistered symbol
    desc = encode_req({:symbol => 'Stuff.sum', :args => [2, 2]})
    get "/api/?r=#{desc}"

    # Then I receive an error message saying that I have no permission to
    # access that
    last_response.status.should be_eql 550
    last_response.headers["content-type"].should be_eql "application/json;charset=utf-8"
    JSON.parse(last_response.body).should be_eql({'error' => "You don't have access to this resource"})
  end

  it "should return the proper status and message when the resource is not found" do
    # When I try to access the unregistered object
    desc = encode_req({:symbol => 'NoSuchObj.no_such_method', :args => [2, 2]})
    get "/api/?r=#{desc}"

    # Then I should see the proper error message raised with a nice message
    last_response.status.should be_eql 404
    last_response.headers["content-type"].should be_eql "application/json;charset=utf-8"
    JSON.parse(last_response.body).should be_eql({'error' => "There's no such object"})
  end

  it "should handle invalid requests with the right status and nice message" do
    # Given that I have a buggy query parameter
    query = 'invalid'

    # When I hit the API end point with that invalid param
    get "/api/?r=#{query}"

    # Then I see that a proper error message was set
    last_response.status.should be_eql 400
    last_response.headers["content-type"].should be_eql "application/json;charset=utf-8"
    JSON.parse(last_response.body).should be_eql(
        {'error' => "The parameter you passed couldn't be parsed"})
  end
end
